from dolfin import *

logging(False)

# Mesh and basis functions (function space) (change mesh for 1D/2D)
#mesh = UnitInterval(100)
mesh = UnitSquare(40, 40)
#mesh = Mesh("dolfin.xml.gz")
V = VectorFunctionSpace(mesh, "CG", 1, 2)

# Coefficient for boundary condition
class Kappa(Expression):
    def eval(self, values, x):
        if(x[0] < 0.5):
            values[0] = 0.0
        else:
            values[0] = 1.0e6

# Define FEM formulation
u = TrialFunction(V) # Linear combination of basis functions phi_j
v = TestFunction(V) # Basis function phi_i
f = Expression("0.0")
k = 1.0e-2
w = 1.0
b = 0.0
kappa = Kappa(V) # Parameter for choosing boundary condition type
u0 = Function(V)
u1 = Function(V)
d0 = Function(V)
d1 = Function(V)
# Define FEM equation for timestep with Trapezoidal method
a = (u[0]*v[0] - 0.5*k*u[1]*v[0] + \
         u[1]*v[1] + 0.5*k*w*inner(grad(u[0]), grad(v[1])))*dx + kappa*(u[0]*v[0] + u[1]*v[1])*ds
L = (u0[0]*v[0] + 0.5*k*u0[1]*v[0] + \
         u0[1]*v[1] - 0.5*k*w*inner(grad(u0[0]), grad(v[1])))*dx
        
# Represent FEM equation
problem = VariationalProblem(a, L)

# Set initial condition
for v in vertices(mesh):
    x = v.point()[0]
    y = v.point()[1]
    r = x
    i = v.index()

    a0 = 1
    b0 = 0.0
    c0 = 0.001

    u1.vector()[i] = a0*exp(-pow(r-b0, 2.0)/(2.0*c0))

file_u = File("wave.pvd")
u1_0 = u1.split()[0]
file_u << u1_0

# Timestepping
T = 2.0
t = 0.0
counter = 0
while(t < T):
    u0.assign(u1)
    # Build linear system and solve for coefficients uh in linear combination
    u1 = problem.solve()

    # Save solution in ParaView/VTK format
    if(counter % 2 == 0):
        print "t: ", t
        u1_0 = u1.split()[0]
        file_u << u1_0

    t += k
    counter += 1

u1_0 = u1.split()[0]
plot(u1_0, interactive=True)
